@@ -7,8 +7,8 @@ use base 'Cache::Memcached::AnyEvent::Protocol';
         my $cmd = shift;
         return sub {
             my ($guard, $self, $memcached, $key, $value, $initial, $cb) = @_;
-            my $fq_key = $self->prepare_key( $key );
-            my $handle = $self->get_handle_for( $fq_key );
+            my $fq_key = $memcached->prepare_key( $key );
+            my $handle = $memcached->get_handle_for( $fq_key );
         
             $value ||= 1;
             my @command = ($cmd => $fq_key => $value);
@@ -44,8 +44,8 @@ sub _build_delete_cb {
     return sub {
         my ($guard, $self, $memcached, $key, $noreply, $cb) = @_;
 
-        my $fq_key = $self->prepare_key( $key );
-        my $handle = $self->get_handle_for( $key );
+        my $fq_key = $memcached->prepare_key( $key );
+        my $handle = $memcached->get_handle_for( $key );
 
         my @command = (delete => $key);
         $noreply = 0; # XXX - FIXME
@@ -90,7 +90,7 @@ sub _build_get_multi_cb {
         my $count = $memcached->get_server_count();
         my @keysinserver;
         foreach my $key (@$keys) {
-            my $fq_key = $self->prepare_key( $key );
+            my $fq_key = $memcached->prepare_key( $key );
             my $hash   = $memcached->{hashing_algorithm}->hash($fq_key);
             my $i      = $hash % $count;
             my $handle = $memcached->get_handle( $memcached->get_server($i) );
@@ -115,8 +115,7 @@ sub _build_get_multi_cb {
                 } elsif ($line =~ /^VALUE (\S+) (\S+) (\S+)(?: (\S+))?/)  {
                     my ($rkey, $rflags, $rsize, $rcas) = ($1, $2, $3, $4);
                     $handle->push_read(chunk => $rsize, sub {
-                        my $key = $self->decode_key($rkey);
-                        my $data = $self->decode_value($rflags, $_[1]);
+                        my ($key, $data) = $memcached->decode_key_value($rkey, $rflags, $_[1]);
                         $rv{ $key } = $data; # XXX whatabout CAS?
                         $handle->push_read(regex => qr{\r\n}, cb => sub { "noop" });
                         $handle->push_read(line => $code);
@@ -137,11 +136,11 @@ sub _build_get_multi_cb {
         my $cmd = shift;
         sub {
             my ($guard, $self, $memcached, $key, $value, $exptime, $noreply, $cb) = @_;
-            my $fq_key = $self->prepare_key( $key );
-            my $handle = $self->get_handle_for( $fq_key );
+            my $fq_key = $memcached->prepare_key( $key );
+            my $handle = $memcached->get_handle_for( $fq_key );
 
             my ($write_data, $write_len, $flags, $expires) =
-                $self->prepare_value( $cmd, $value, $exptime );
+                $memcached->prepare_value( $cmd, $value, $exptime );
             $handle->push_write("$cmd $fq_key $flags $expires $write_len\r\n$write_data\r\n");
             if (! $noreply) {
                 $handle->push_read(regex => qr{^STORED\r\n}, sub {
